@@ -1,9 +1,13 @@
 import inspect
 
 try:
-    from contextlib import asynccontextmanager  # noqa: F401 lgtm[py/unused-import]
+    from contextlib import (  # noqa: F401 lgtm[py/unused-import]
+        asynccontextmanager,
+    )
 except ImportError:
-    from async_generator import asynccontextmanager  # noqa: F401 E501, lgtm[py/unused-import]
+    from async_generator import (  # noqa: F401 E501, lgtm[py/unused-import]
+        asynccontextmanager,
+    )
 
 
 async def resolve_awaitable(obj):
