@@ -1,14 +1,23 @@
 import asyncio
 
-from botocore.endpoint import EndpointCreator, Endpoint, DEFAULT_TIMEOUT, \
-    MAX_POOL_CONNECTIONS, logger, history_recorder, create_request_object, \
-    is_valid_ipv6_endpoint_url, is_valid_endpoint_url, HTTPClientError
+from botocore.endpoint import (
+    DEFAULT_TIMEOUT,
+    MAX_POOL_CONNECTIONS,
+    Endpoint,
+    EndpointCreator,
+    HTTPClientError,
+    create_request_object,
+    history_recorder,
+    is_valid_endpoint_url,
+    is_valid_ipv6_endpoint_url,
+    logger,
+)
 from botocore.hooks import first_non_none_response
 from urllib3.response import HTTPHeaderDict
 
+from aiobotocore.httpchecksum import handle_checksum_body
 from aiobotocore.httpsession import AIOHTTPSession
 from aiobotocore.response import StreamingBody
-from aiobotocore.httpchecksum import handle_checksum_body
 
 
 async def convert_to_response_dict(http_response, operation_model):
@@ -33,12 +42,15 @@ async def convert_to_response_dict(http_response, operation_model):
         # https://github.com/aio-libs/aiobotocore/pull/116
         # aiohttp's CIMultiDict camel cases the headers :(
         'headers': HTTPHeaderDict(
-            {k.decode('utf-8').lower(): v.decode('utf-8')
-             for k, v in http_response.raw.raw_headers}),
+            {
+                k.decode('utf-8').lower(): v.decode('utf-8')
+                for k, v in http_response.raw.raw_headers
+            }
+        ),
         'status_code': http_response.status_code,
         'context': {
             'operation_name': operation_model.name,
-        }
+        },
     }
     if response_dict['status_code'] >= 300:
         response_dict['body'] = await http_response.content
@@ -53,19 +65,27 @@ async def convert_to_response_dict(http_response, operation_model):
 
 
 class AioEndpoint(Endpoint):
+    async def close(self):
+        await self.http_session.close()
+
     async def create_request(self, params, operation_model=None):
         request = create_request_object(params)
         if operation_model:
-            request.stream_output = any([
-                operation_model.has_streaming_output,
-                operation_model.has_event_stream_output
-            ])
+            request.stream_output = any(
+                [
+                    operation_model.has_streaming_output,
+                    operation_model.has_event_stream_output,
+                ]
+            )
             service_id = operation_model.service_model.service_id.hyphenize()
             event_name = 'request-created.{service_id}.{op_name}'.format(
-                service_id=service_id,
-                op_name=operation_model.name)
-            await self._event_emitter.emit(event_name, request=request,
-                                           operation_name=operation_model.name)
+                service_id=service_id, op_name=operation_model.name
+            )
+            await self._event_emitter.emit(
+                event_name,
+                request=request,
+                operation_name=operation_model.name,
+            )
         prepared_request = self.prepare_request(request)
         return prepared_request
 
@@ -74,31 +94,37 @@ class AioEndpoint(Endpoint):
         context = request_dict['context']
         self._update_retries_context(context, attempts)
         request = await self.create_request(request_dict, operation_model)
-        context = request_dict['context']
         success_response, exception = await self._get_response(
-            request, operation_model, context)
-        while await self._needs_retry(attempts, operation_model,
-                                      request_dict, success_response,
-                                      exception):
+            request, operation_model, context
+        )
+        while await self._needs_retry(
+            attempts,
+            operation_model,
+            request_dict,
+            success_response,
+            exception,
+        ):
             attempts += 1
-            self._update_retries_context(
-                context, attempts, success_response
-            )
+            self._update_retries_context(context, attempts, success_response)
             # If there is a stream associated with the request, we need
             # to reset it before attempting to send the request again.
             # This will ensure that we resend the entire contents of the
             # body.
             request.reset_stream()
             # Create a new request when retried (including a new signature).
-            request = await self.create_request(
-                request_dict, operation_model)
+            request = await self.create_request(request_dict, operation_model)
             success_response, exception = await self._get_response(
-                request, operation_model, context)
-        if success_response is not None and \
-                'ResponseMetadata' in success_response[1]:
+                request, operation_model, context
+            )
+        if (
+            success_response is not None
+            and 'ResponseMetadata' in success_response[1]
+        ):
             # We want to share num retries, not num attempts.
             total_retries = attempts - 1
-            success_response[1]['ResponseMetadata']['RetryAttempts'] = total_retries
+            success_response[1]['ResponseMetadata'][
+                'RetryAttempts'
+            ] = total_retries
         if exception is not None:
             raise exception
         else:
@@ -111,7 +137,8 @@ class AioEndpoint(Endpoint):
         # If an exception occurs then the success_response is None.
         # If no exception occurs then exception is None.
         success_response, exception = await self._do_get_response(
-            request, operation_model, context)
+            request, operation_model, context
+        )
         kwargs_to_emit = {
             'response_dict': None,
             'parsed_response': None,
@@ -122,48 +149,59 @@ class AioEndpoint(Endpoint):
             http_response, parsed_response = success_response
             kwargs_to_emit['parsed_response'] = parsed_response
             kwargs_to_emit['response_dict'] = await convert_to_response_dict(
-                http_response, operation_model)
+                http_response, operation_model
+            )
         service_id = operation_model.service_model.service_id.hyphenize()
         await self._event_emitter.emit(
-            'response-received.%s.%s' % (
-                service_id, operation_model.name), **kwargs_to_emit)
+            f"response-received.{service_id}.{operation_model.name}",
+            **kwargs_to_emit,
+        )
         return success_response, exception
 
     async def _do_get_response(self, request, operation_model, context):
         try:
             logger.debug("Sending http request: %s", request)
-            history_recorder.record('HTTP_REQUEST', {
-                'method': request.method,
-                'headers': request.headers,
-                'streaming': operation_model.has_streaming_input,
-                'url': request.url,
-                'body': request.body
-            })
+            history_recorder.record(
+                'HTTP_REQUEST',
+                {
+                    'method': request.method,
+                    'headers': request.headers,
+                    'streaming': operation_model.has_streaming_input,
+                    'url': request.url,
+                    'body': request.body,
+                },
+            )
             service_id = operation_model.service_model.service_id.hyphenize()
-            event_name = 'before-send.%s.%s' % (
-                service_id, operation_model.name)
-            responses = await self._event_emitter.emit(event_name,
-                                                       request=request)
+            event_name = f"before-send.{service_id}.{operation_model.name}"
+            responses = await self._event_emitter.emit(
+                event_name, request=request
+            )
             http_response = first_non_none_response(responses)
             if http_response is None:
                 http_response = await self._send(request)
         except HTTPClientError as e:
-            return None, e
+            return (None, e)
         except Exception as e:
-            logger.debug("Exception received when sending HTTP request.",
-                         exc_info=True)
-            return None, e
+            logger.debug(
+                "Exception received when sending HTTP request.", exc_info=True
+            )
+            return (None, e)
 
         # This returns the http_response and the parsed_data.
-        response_dict = await convert_to_response_dict(http_response,
-                                                       operation_model)
+        response_dict = await convert_to_response_dict(
+            http_response, operation_model
+        )
         await handle_checksum_body(
-            http_response, response_dict, context, operation_model,
+            http_response,
+            response_dict,
+            context,
+            operation_model,
         )
 
         http_response_record_dict = response_dict.copy()
-        http_response_record_dict['streaming'] = \
-            operation_model.has_streaming_output
+        http_response_record_dict[
+            'streaming'
+        ] = operation_model.has_streaming_output
         history_recorder.record('HTTP_RESPONSE', http_response_record_dict)
 
         protocol = operation_model.metadata['protocol']
@@ -171,22 +209,29 @@ class AioEndpoint(Endpoint):
 
         if asyncio.iscoroutinefunction(parser.parse):
             parsed_response = await parser.parse(
-                response_dict, operation_model.output_shape)
+                response_dict, operation_model.output_shape
+            )
         else:
             parsed_response = parser.parse(
-                response_dict, operation_model.output_shape)
+                response_dict, operation_model.output_shape
+            )
 
         if http_response.status_code >= 300:
             await self._add_modeled_error_fields(
-                response_dict, parsed_response,
-                operation_model, parser,
+                response_dict,
+                parsed_response,
+                operation_model,
+                parser,
             )
         history_recorder.record('PARSED_RESPONSE', parsed_response)
         return (http_response, parsed_response), None
 
     async def _add_modeled_error_fields(
-            self, response_dict, parsed_response,
-            operation_model, parser,
+        self,
+        response_dict,
+        parsed_response,
+        operation_model,
+        parser,
     ):
         error_code = parsed_response.get("Error", {}).get("Code")
         if error_code is None:
@@ -204,24 +249,35 @@ class AioEndpoint(Endpoint):
         parsed_response.update(modeled_parse)
 
     # NOTE: The only line changed here changing time.sleep to asyncio.sleep
-    async def _needs_retry(self, attempts, operation_model, request_dict,
-                           response=None, caught_exception=None):
+    async def _needs_retry(
+        self,
+        attempts,
+        operation_model,
+        request_dict,
+        response=None,
+        caught_exception=None,
+    ):
         service_id = operation_model.service_model.service_id.hyphenize()
-        event_name = 'needs-retry.%s.%s' % (
-            service_id,
-            operation_model.name)
+        event_name = f"needs-retry.{service_id}.{operation_model.name}"
         responses = await self._event_emitter.emit(
-            event_name, response=response, endpoint=self,
-            operation=operation_model, attempts=attempts,
-            caught_exception=caught_exception, request_dict=request_dict)
+            event_name,
+            response=response,
+            endpoint=self,
+            operation=operation_model,
+            attempts=attempts,
+            caught_exception=caught_exception,
+            request_dict=request_dict,
+        )
         handler_response = first_non_none_response(responses)
         if handler_response is None:
             return False
         else:
             # Request needs to be retried, and we need to sleep
             # for the specified number of times.
-            logger.debug("Response received to retry, sleeping for "
-                         "%s seconds", handler_response)
+            logger.debug(
+                "Response received to retry, sleeping for %s seconds",
+                handler_response,
+            )
             await asyncio.sleep(handler_response)
             return True
 
@@ -231,20 +287,24 @@ class AioEndpoint(Endpoint):
 
 class AioEndpointCreator(EndpointCreator):
     def create_endpoint(
-            self, service_model, region_name, endpoint_url,
-            verify=None, response_parser_factory=None,
-            timeout=DEFAULT_TIMEOUT,
-            max_pool_connections=MAX_POOL_CONNECTIONS,
-            http_session_cls=AIOHTTPSession,
-            proxies=None,
-            socket_options=None,
-            client_cert=None, proxies_config=None,
-            connector_args=None
+        self,
+        service_model,
+        region_name,
+        endpoint_url,
+        verify=None,
+        response_parser_factory=None,
+        timeout=DEFAULT_TIMEOUT,
+        max_pool_connections=MAX_POOL_CONNECTIONS,
+        http_session_cls=AIOHTTPSession,
+        proxies=None,
+        socket_options=None,
+        client_cert=None,
+        proxies_config=None,
+        connector_args=None,
     ):
-        if (
-            not is_valid_endpoint_url(endpoint_url)
-            and not is_valid_ipv6_endpoint_url(endpoint_url)
-        ):
+        if not is_valid_endpoint_url(
+            endpoint_url
+        ) and not is_valid_ipv6_endpoint_url(endpoint_url):
             raise ValueError("Invalid endpoint: %s" % endpoint_url)
 
         if proxies is None:
@@ -260,7 +320,7 @@ class AioEndpointCreator(EndpointCreator):
             socket_options=socket_options,
             client_cert=client_cert,
             proxies_config=proxies_config,
-            connector_args=connector_args
+            connector_args=connector_args,
         )
 
         return AioEndpoint(
@@ -268,5 +328,5 @@ class AioEndpointCreator(EndpointCreator):
             endpoint_prefix=endpoint_prefix,
             event_emitter=self._event_emitter,
             response_parser_factory=response_parser_factory,
-            http_session=http_session
+            http_session=http_session,
         )
